@@ -140,22 +140,26 @@ gth_image_svg_new (void)
 }
 
 
-static void
+static gboolean
 gth_image_svg_set_handle (GthImageSvg *self,
 			  RsvgHandle  *rsvg)
 {
 	RsvgDimensionData dimension_data;
 
 	if (self->rsvg == rsvg)
-		return;
+		return TRUE;
+
+	rsvg_handle_get_dimensions (rsvg, &dimension_data);
+	if ((dimension_data.width == 0) || (dimension_data.height == 0))
+		return FALSE;
 
 	self->rsvg = g_object_ref (rsvg);
-
-	rsvg_handle_get_dimensions (self->rsvg, &dimension_data);
 	self->original_width = dimension_data.width;
 	self->original_height = dimension_data.height;
 
 	gth_image_svg_set_zoom (GTH_IMAGE (self), 1.0, NULL, NULL);
+
+	return TRUE;
 }
 
 
@@ -183,7 +187,11 @@ _cairo_image_surface_create_from_svg (GInputStream  *istream,
 						 cancellable,
 						 error);
 	if (rsvg != NULL) {
-		gth_image_svg_set_handle (GTH_IMAGE_SVG (image), rsvg);
+		if (! gth_image_svg_set_handle (GTH_IMAGE_SVG (image), rsvg)) {
+			g_set_error (error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "Error");
+			g_object_unref (image);
+			image = NULL;
+		}
 		g_object_unref (rsvg);
 	}
 
