@@ -28,8 +28,6 @@
 struct _GthEditCommentDialogPrivate {
 	GtkWidget *notebook;
 	GtkWidget *save_changed_checkbutton;
-	GtkWidget *keep_open_check_button;
-	GtkWidget *info;
 };
 
 
@@ -54,8 +52,6 @@ gth_edit_comment_dialog_set_file_list (GthEditMetadataDialog *base,
 	GList                *pages;
 	GList                *scan;
 
-	gth_file_selection_info_set_file_list (GTH_FILE_SELECTION_INFO (self->priv->info), file_list);
-
 	/* update the widgets */
 
 	n_files = g_list_length (file_list);
@@ -101,20 +97,11 @@ gth_edit_comment_dialog_update_info (GthEditMetadataDialog *base,
 }
 
 
-static gboolean
-gth_edit_comment_dialog_get_keep_open (GthEditMetadataDialog *base)
-{
-	GthEditCommentDialog *self = GTH_EDIT_COMMENT_DIALOG (base);
-	return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->priv->keep_open_check_button));
-}
-
-
 static void
 gth_edit_comment_dialog_gth_edit_metadata_dialog_interface_init (GthEditMetadataDialogInterface *iface)
 {
 	iface->set_file_list = gth_edit_comment_dialog_set_file_list;
 	iface->update_info = gth_edit_comment_dialog_update_info;
-	iface->get_keep_open = gth_edit_comment_dialog_get_keep_open;
 }
 
 
@@ -142,11 +129,7 @@ gth_edit_comment_dialog_init (GthEditCommentDialog *self)
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), vbox, TRUE, TRUE, 0);
-
-	self->priv->info = gth_file_selection_info_new ();
-	gtk_widget_show (self->priv->info);
-	gtk_box_pack_start (GTK_BOX (vbox), self->priv->info, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), vbox, TRUE, TRUE, 0);
 
 	self->priv->notebook = gtk_notebook_new ();
 	gtk_widget_show (self->priv->notebook);
@@ -156,10 +139,6 @@ gth_edit_comment_dialog_init (GthEditCommentDialog *self)
 	gtk_widget_show (self->priv->save_changed_checkbutton);
 	gtk_box_pack_start (GTK_BOX (vbox), self->priv->save_changed_checkbutton, FALSE, FALSE, 0);
 
-	self->priv->keep_open_check_button = gtk_check_button_new_with_mnemonic (_("_Keep the dialog open"));
-	gtk_widget_show (self->priv->keep_open_check_button);
-	gtk_box_pack_start (GTK_BOX (vbox), self->priv->keep_open_check_button, FALSE, FALSE, 0);
-
 	pages = gth_main_get_type_set ("edit-comment-dialog-page");
 	if (pages == NULL)
 		return;
