@@ -449,7 +449,7 @@ set_file_info_from_hash (GFileInfo  *info,
 }
 
 
-static void
+static gboolean
 set_attribute_from_metadata (GFileInfo  *info,
 			     const char *attribute,
 			     GObject    *metadata)
@@ -460,7 +460,7 @@ set_attribute_from_metadata (GFileInfo  *info,
 	char *type_name;
 
 	if (metadata == NULL)
-		return;
+		return FALSE;
 
 	g_object_get (metadata,
 		      "description", &description,
@@ -481,6 +481,8 @@ set_attribute_from_metadata (GFileInfo  *info,
 	g_free (formatted_value);
 	g_free (raw_value);
 	g_free (type_name);
+
+	return TRUE;
 }
 
 
@@ -502,20 +504,17 @@ get_attribute_from_tagset (GFileInfo  *info,
 }
 
 
-static void
+static gboolean
 set_attribute_from_tagset (GFileInfo  *info,
 			   const char *attribute,
 			   const char *tagset[])
 {
-	GObject *metadata;
-
-	metadata = get_attribute_from_tagset (info, tagset);
-	if (metadata != NULL)
-		set_attribute_from_metadata (info, attribute, metadata);
+	GObject *metadata = get_attribute_from_tagset (info, tagset);
+	return set_attribute_from_metadata (info, attribute, metadata);
 }
 
 
-static void
+static gboolean
 set_string_list_attribute_from_tagset (GFileInfo  *info,
 				       const char *attribute,
 				       const char *tagset[])
@@ -531,7 +530,7 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 	}
 
 	if (metadata == NULL)
-		return;
+		return FALSE;
 
 	if (GTH_IS_METADATA (metadata) && (gth_metadata_get_data_type (GTH_METADATA (metadata)) != GTH_METADATA_TYPE_STRING_LIST)) {
 		char           *raw;
@@ -541,8 +540,9 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 
 		g_object_get (metadata, "raw", &raw, NULL);
 		utf8_raw = _g_utf8_try_from_any (raw);
-		if (utf8_raw == NULL)
-			return;
+		if (utf8_raw == NULL) {
+			return FALSE;
+		}
 
 		keywords = g_strsplit (utf8_raw, ",", -1);
 		string_list = gth_string_list_new_from_strv (keywords);
@@ -555,8 +555,10 @@ set_string_list_attribute_from_tagset (GFileInfo  *info,
 		g_free (raw);
 		g_free (utf8_raw);
 	}
-	else
+	else {
 		g_file_info_set_attribute_object (info, attribute, metadata);
+	}
+	return TRUE;
 }
 
 
@@ -590,9 +592,17 @@ extern "C"
 void
 exiv2_update_general_attributes (GFileInfo *info)
 {
-	set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES);
-	set_attribute_from_tagset (info, "general::description", _DESCRIPTION_TAG_NAMES);
-	set_attribute_from_tagset (info, "general::title", _TITLE_TAG_NAMES);
+	gboolean updated = FALSE;
+
+	if (set_attribute_from_tagset (info, "general::datetime", _ORIGINAL_DATE_TAG_NAMES)) {
+		updated = TRUE;
+	}
+	if (set_attribute_from_tagset (info, "general::description", _DESCRIPTION_TAG_NAMES)) {
+		updated = TRUE;
+	}
+	if (set_attribute_from_tagset (info, "general::title", _TITLE_TAG_NAMES)) {
+		updated = TRUE;
+	}
 
 	/* if iptc::caption and iptc::headline are different use iptc::headline
 	 * to set general::title, if not already set. */
@@ -609,13 +619,23 @@ exiv2_update_general_attributes (GFileInfo *info)
 		    && (g_strcmp0 (gth_metadata_get_raw (GTH_METADATA (iptc_caption)),
 				   gth_metadata_get_raw (GTH_METADATA (iptc_headline))) != 0))
 		{
-			set_attribute_from_metadata (info, "general::title", iptc_headline);
+			if (set_attribute_from_metadata (info, "general::title", iptc_headline)) {
+				updated = TRUE;
+			}
 		}
 	}
 
-	set_attribute_from_tagset (info, "general::location", _LOCATION_TAG_NAMES);
-	set_string_list_attribute_from_tagset (info, "general::tags", _KEYWORDS_TAG_NAMES);
-	set_attribute_from_tagset (info, "general::rating", _RATING_TAG_NAMES);
+	if (set_attribute_from_tagset (info, "general::location", _LOCATION_TAG_NAMES)) {
+		updated = TRUE;
+	}
+	if (set_string_list_attribute_from_tagset (info, "general::tags", _KEYWORDS_TAG_NAMES)) {
+		updated = TRUE;
+	}
+	if (set_attribute_from_tagset (info, "general::rating", _RATING_TAG_NAMES)) {
+		updated = TRUE;
+	}
+
+	g_file_info_set_attribute_boolean (info, "embedded::updated-general-attributes", updated);
 }
 
 
@@ -855,64 +875,6 @@ exiv2_read_metadata (Exiv2::Image::AutoPtr  image,
 }
 
 
-/*
- * exiv2_read_metadata_from_file
- * reads metadata from image files
- * code relies heavily on example1 from the exiv2 website
- * http://www.exiv2.org/example1.html
- */
-extern "C"
-gboolean
-exiv2_read_metadata_from_file (GFile         *file,
-			       GFileInfo     *info,
-			       gboolean       update_general_attributes,
-			       GCancellable  *cancellable,
-			       GError       **error)
-{
-	try {
-		char *path;
-
-		path = g_file_get_path (file);
-		if (path == NULL) {
-			if (error != NULL)
-				*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
-			return FALSE;
-		}
-
-#if EXIV2_TEST_VERSION(0,28,0)
-		Exiv2::Image::UniquePtr image = Exiv2::ImageFactory::open(path);
-#else
-		Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open(path);
-#endif
-		g_free (path);
-
-		if (image.get() == 0) {
-			if (error != NULL)
-				*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
-			return FALSE;
-		}
-		// Set the log level to only show errors (and suppress warnings, informational and debug messages)
-		Exiv2::LogMsg::setLevel(Exiv2::LogMsg::error);
-#if EXIV2_TEST_VERSION(0,28,0)
-		exiv2_read_metadata (std::move(image), info, update_general_attributes);
-#else
-		exiv2_read_metadata (image, info, update_general_attributes);
-#endif
-	}
-#if EXIV2_TEST_VERSION(0,28,0)
-	catch (Exiv2::Error& e) {
-#else
-	catch (Exiv2::AnyError& e) {
-#endif
-		if (error != NULL)
-			*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, e.what());
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-
 extern "C"
 gboolean
 exiv2_read_metadata_from_buffer (void       *buffer,
@@ -933,7 +895,8 @@ exiv2_read_metadata_from_buffer (void       *buffer,
 				*error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, _("Invalid file format"));
 			return FALSE;
 		}
-
+		// Set the log level to only show errors (and suppress warnings, informational and debug messages)
+		Exiv2::LogMsg::setLevel(Exiv2::LogMsg::error);
 #if EXIV2_TEST_VERSION(0,28,0)
 		exiv2_read_metadata (std::move(image), info, update_general_attributes);
 #else
@@ -1226,7 +1189,6 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 	mandatory_int (ed, "Exif.Image.YResolution", 72);
 	mandatory_int (ed, "Exif.Image.ResolutionUnit", 2);
 	mandatory_int (ed, "Exif.Image.YCbCrPositioning", 1);
-	mandatory_int (ed, "Exif.Photo.ColorSpace", 1);
 	mandatory_string (ed, "Exif.Photo.ExifVersion", "48 50 50 49");
 	mandatory_string (ed, "Exif.Photo.ComponentsConfiguration", "1 2 3 0");
 	mandatory_string (ed, "Exif.Photo.FlashpixVersion", "48 49 48 48");
@@ -1290,14 +1252,16 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 			ed["Exif.Thumbnail.ResolutionUnit"] =  2;
 			g_free (buffer);
 		}
-		else
+		else {
 			thumb.erase();
+		}
 
 		g_object_unref (thumbnail_data);
 		cairo_surface_destroy (thumbnail);
 	}
-	else
+	else if (g_file_info_get_attribute_boolean (info, "gth::file::image-changed")) {
 		thumb.erase();
+	}
 
 	if (surface != NULL)
 		cairo_surface_destroy (surface);
@@ -1349,9 +1313,10 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 					GthStringList *string_list = gth_metadata_get_string_list (GTH_METADATA (metadatum));
 					for (GList *scan = gth_string_list_get_list (string_list); scan; scan = scan->next) {
 						char *single_value = (char *) scan->data;
-
-						value->read (single_value);
-						id.add (iptc_key, value.get());
+						if ((single_value != NULL) && (strcmp (single_value, "") != 0)) {
+							value->read (single_value);
+							id.add (iptc_key, value.get());
+						}
 					}
 					break;
 				}
@@ -1407,9 +1372,10 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 					GthStringList *string_list = gth_metadata_get_string_list (GTH_METADATA (metadatum));
 					for (GList *scan = gth_string_list_get_list (string_list); scan; scan = scan->next) {
 						char *single_value = (char *) scan->data;
-
-						value->read (single_value);
-						xd.add (xmp_key, value.get());
+						if ((single_value != NULL) && (strcmp (single_value, "") != 0)) {
+							value->read (single_value);
+							xd.add (xmp_key, value.get());
+						}
 					}
 					break;
 				}
@@ -1450,58 +1416,138 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 }
 
 
-extern "C"
-gboolean
-exiv2_supports_writes (const char *mime_type)
-{
-	return (g_content_type_equals (mime_type, "image/jpeg")
-		|| g_content_type_equals (mime_type, "image/tiff")
-		|| g_content_type_equals (mime_type, "image/png")
-		|| g_content_type_equals (mime_type, "image/webp"));
-}
+void update_exif_tags_from_general_attributes (GFileInfo *info) {
+	GObject *metadata = g_file_info_get_attribute_object (info, "general::description");
+	if (metadata != NULL) {
+		const char *tags_to_remove[] = {
+			"Exif::Image::ImageDescription",
+			"Xmp::tiff::ImageDescription",
+			"Iptc::Application2::Headline",
+			NULL
+		};
+		const char *tags_to_update[] = {
+			"Exif::Photo::UserComment",
+			"Xmp::dc::description",
+			"Iptc::Application2::Caption",
+			NULL
+		};
 
+		for (int i = 0; tags_to_remove[i] != NULL; i++)
+			g_file_info_remove_attribute (info, tags_to_remove[i]);
 
-extern "C"
-gboolean
-exiv2_write_metadata (GthImageSaveData *data)
-{
-	if (exiv2_supports_writes (data->mime_type) && (data->file_data != NULL)) {
-		try {
-#if EXIV2_TEST_VERSION(0,28,0)
-			Exiv2::Image::UniquePtr image = Exiv2::ImageFactory::open ((Exiv2::byte*) data->buffer, data->buffer_size);
-#else
-			Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open ((Exiv2::byte*) data->buffer, data->buffer_size);
-#endif
-			g_assert (image.get() != 0);
+		/* Remove the value type to use the default type for each field
+		 * as described in exiv2_tools/main.c */
 
-#if EXIV2_TEST_VERSION(0,28,0)
-			Exiv2::DataBuf buf = exiv2_write_metadata_private (std::move(image), data->file_data->info, data->image);
-#else
-			Exiv2::DataBuf buf = exiv2_write_metadata_private (image, data->file_data->info, data->image);
-#endif
+		g_object_set (metadata, "value-type", NULL, NULL);
 
-			g_free (data->buffer);
-#if EXIV2_TEST_VERSION(0,28,0)
-			data->buffer = g_memdup (buf.data(), buf.size());
-			data->buffer_size = buf.size();
-#else
-			data->buffer = g_memdup (buf.pData_, buf.size_);
-			data->buffer_size = buf.size_;
-#endif
+		for (int i = 0; tags_to_update[i] != NULL; i++) {
+			GObject *orig_metadata;
+
+			orig_metadata = g_file_info_get_attribute_object (info, tags_to_update[i]);
+			if (orig_metadata != NULL) {
+				/* keep the original value type */
+
+				g_object_set (orig_metadata,
+					      "raw", gth_metadata_get_raw (GTH_METADATA (metadata)),
+					      "formatted", gth_metadata_get_formatted (GTH_METADATA (metadata)),
+					      NULL);
+			}
+			else
+				g_file_info_set_attribute_object (info, tags_to_update[i], metadata);
 		}
-#if EXIV2_TEST_VERSION(0,28,0)
-		catch (Exiv2::Error& e) {
-#else
-		catch (Exiv2::AnyError& e) {
-#endif
-			if (data->error != NULL)
-				*data->error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_FAILED, e.what());
-			g_warning ("%s\n", e.what());
-			return FALSE;
+	}
+	else {
+		for (int i = 0; _DESCRIPTION_TAG_NAMES[i] != NULL; i++)
+			g_file_info_remove_attribute (info, _DESCRIPTION_TAG_NAMES[i]);
+	}
+
+	metadata = g_file_info_get_attribute_object (info, "general::title");
+	if (metadata != NULL) {
+		g_object_set (metadata, "value-type", NULL, NULL);
+		for (int i = 0; _TITLE_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_set_attribute_object (info, _TITLE_TAG_NAMES[i], metadata);
+		}
+	}
+	else {
+		for (int i = 0; _TITLE_TAG_NAMES[i] != NULL; i++)
+			g_file_info_remove_attribute (info, _TITLE_TAG_NAMES[i]);
+	}
+
+	metadata = g_file_info_get_attribute_object (info, "general::location");
+	if (metadata != NULL) {
+		g_object_set (metadata, "value-type", NULL, NULL);
+		for (int i = 0; _LOCATION_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_set_attribute_object (info, _LOCATION_TAG_NAMES[i], metadata);
+		}
+	}
+	else {
+		for (int i = 0; _LOCATION_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_remove_attribute (info, _LOCATION_TAG_NAMES[i]);
 		}
 	}
 
-	return TRUE;
+	metadata = g_file_info_get_attribute_object (info, "general::tags");
+	if (metadata != NULL) {
+		if (GTH_IS_METADATA (metadata))
+			g_object_set (metadata, "value-type", NULL, NULL);
+		for (int i = 0; _KEYWORDS_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_set_attribute_object (info, _KEYWORDS_TAG_NAMES[i], metadata);
+		}
+	}
+	else {
+		for (int i = 0; _KEYWORDS_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_remove_attribute (info, _KEYWORDS_TAG_NAMES[i]);
+		}
+	}
+
+	metadata = g_file_info_get_attribute_object (info, "general::rating");
+	if (metadata != NULL) {
+		if (GTH_IS_METADATA (metadata))
+			g_object_set (metadata, "value-type", NULL, NULL);
+		for (int i = 0; _RATING_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_set_attribute_object (info, _RATING_TAG_NAMES[i], metadata);
+		}
+	}
+	else {
+		for (int i = 0; _RATING_TAG_NAMES[i] != NULL; i++) {
+			g_file_info_remove_attribute (info, _RATING_TAG_NAMES[i]);
+		}
+	}
+
+	metadata = g_file_info_get_attribute_object (info, "general::datetime");
+	if (metadata != NULL) {
+		GthMetadata *xmp_metadata = NULL;
+		GTimeVal     timeval;
+
+		if (_g_time_val_from_exif_date (gth_metadata_get_raw (GTH_METADATA (metadata)), &timeval)) {
+			char *xmp_format;
+
+			xmp_metadata = gth_metadata_new ();
+			xmp_format = _g_time_val_to_xmp_date (&timeval);
+			g_object_set (xmp_metadata,
+				      "raw", xmp_format,
+				      "formatted", gth_metadata_get_formatted (GTH_METADATA (metadata)),
+				      "value-type", NULL, /* use the default type as described in extensions/exiv2_tools/main.c */
+				      NULL);
+
+			g_free (xmp_format);
+		}
+
+		for (int i = 0; _ORIGINAL_DATE_TAG_NAMES[i] != NULL; i++) {
+			if (g_str_has_prefix (_ORIGINAL_DATE_TAG_NAMES[i], "Xmp::")) {
+				if (xmp_metadata != NULL)
+					g_file_info_set_attribute_object (info, _ORIGINAL_DATE_TAG_NAMES[i], G_OBJECT (xmp_metadata));
+			}
+			else
+				g_file_info_set_attribute_object (info, _ORIGINAL_DATE_TAG_NAMES[i], metadata);
+		}
+
+		_g_object_unref (xmp_metadata);
+	}
+	else {
+		for (int i = 0; _ORIGINAL_DATE_TAG_NAMES[i] != NULL; i++)
+			g_file_info_remove_attribute (info, _ORIGINAL_DATE_TAG_NAMES[i]);
+	}
 }
 
 
@@ -1514,6 +1560,8 @@ exiv2_write_metadata_to_buffer (void      **buffer,
 				GError    **error)
 {
 	try {
+		update_exif_tags_from_general_attributes (info);
+
 #if EXIV2_TEST_VERSION(0,28,0)
 		Exiv2::Image::UniquePtr image = Exiv2::ImageFactory::open ((Exiv2::byte*) *buffer, *buffer_size);
 #else
@@ -1737,3 +1785,23 @@ exiv2_generate_thumbnail (const char *uri,
 
 	return pixbuf;
 }
+
+
+void
+update_exif_dimensions (GFileInfo    *info,
+		        GthTransform  transform)
+{
+	g_return_if_fail (info != NULL);
+
+	if ((transform == GTH_TRANSFORM_ROTATE_90)
+	    || (transform == GTH_TRANSFORM_ROTATE_270)
+	    || (transform == GTH_TRANSFORM_TRANSPOSE)
+	    || (transform == GTH_TRANSFORM_TRANSVERSE))
+	{
+		_g_file_info_swap_attributes (info, "Exif::Photo::PixelXDimension", "Exif::Photo::PixelYDimension");
+		_g_file_info_swap_attributes (info, "Exif::Image::XResolution", "Exif::Image::YResolution");
+		_g_file_info_swap_attributes (info, "Exif::Photo::FocalPlaneXResolution", "Exif::Photo::FocalPlaneYResolution");
+		_g_file_info_swap_attributes (info, "Exif::Image::ImageWidth", "Exif::Image::ImageLength");
+		_g_file_info_swap_attributes (info, "Exif::Iop::RelatedImageWidth", "Exif::Iop::RelatedImageLength");
+	}
+}
