@@ -32,10 +32,10 @@
 
 /* Properties */
 enum {
-        PROP_0,
-        PROP_HISTOGRAM,
-        PROP_CURRENT_CHANNEL,
-        PROP_SCALE_TYPE
+	PROP_0,
+	PROP_HISTOGRAM,
+	PROP_CURRENT_CHANNEL,
+	PROP_SCALE_TYPE
 };
 
 enum {
@@ -46,8 +46,8 @@ enum {
 
 /* Signals */
 enum {
-        CHANGED,
-        LAST_SIGNAL
+	CHANGED,
+	LAST_SIGNAL
 };
 
 
@@ -73,7 +73,10 @@ struct _GthCurveEditorPrivate {
 static guint gth_curve_editor_signals[LAST_SIGNAL] = { 0 };
 
 
-G_DEFINE_TYPE (GthCurveEditor, gth_curve_editor, GTK_TYPE_BOX)
+G_DEFINE_TYPE_WITH_CODE (GthCurveEditor,
+			 gth_curve_editor,
+			 GTK_TYPE_BOX,
+			 G_ADD_PRIVATE (GthCurveEditor))
 
 
 static void
@@ -153,8 +156,6 @@ gth_curve_editor_class_init (GthCurveEditorClass *klass)
 {
 	GObjectClass   *object_class;
 
-	g_type_class_add_private (klass, sizeof (GthCurveEditorPrivate));
-
 	object_class = (GObjectClass*) klass;
 	object_class->set_property = gth_curve_editor_set_property;
 	object_class->get_property = gth_curve_editor_get_property;
@@ -650,7 +651,7 @@ curve_editor_button_press_event_cb (GtkWidget      *widget,
 
 			self->priv->dragging = TRUE;
 
-			cursor = gdk_cursor_new_for_display (gtk_widget_get_display (self->priv->view), GDK_BLANK_CURSOR);
+			cursor = _gdk_cursor_new_for_widget (self->priv->view, GDK_BLANK_CURSOR);
 			gdk_window_set_cursor (gtk_widget_get_window (self->priv->view), cursor);
 			g_object_unref (cursor);
 		}
@@ -689,7 +690,7 @@ curve_editor_button_release_event_cb (GtkWidget      *widget,
 	if (self->priv->dragging) {
 		GdkCursor *cursor;
 
-		cursor = gdk_cursor_new_for_display (gtk_widget_get_display (self->priv->view), GDK_CROSSHAIR);
+		cursor = _gdk_cursor_new_for_widget (self->priv->view, GDK_CROSSHAIR);
 		gdk_window_set_cursor (gtk_widget_get_window (self->priv->view), cursor);
 		g_object_unref (cursor);
 	}
@@ -754,7 +755,7 @@ curve_editor_realize_cb (GtkWidget *widget,
 	GthCurveEditor *self = user_data;
 	GdkCursor      *cursor;
 
-	cursor = gdk_cursor_new_for_display (gtk_widget_get_display (self->priv->view), GDK_CROSSHAIR);
+	cursor = _gdk_cursor_new_for_widget (self->priv->view, GDK_CROSSHAIR);
 	gdk_window_set_cursor (gtk_widget_get_window (self->priv->view), cursor);
 	g_object_unref (cursor);
 }
@@ -864,7 +865,7 @@ gth_curve_editor_init (GthCurveEditor *self)
 	GtkWidget       *button;
 	int              c;
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_CURVE_EDITOR, GthCurveEditorPrivate);
+	self->priv = gth_curve_editor_get_instance_private (self);
 	self->priv->histogram = NULL;
 	self->priv->current_channel = GTH_HISTOGRAM_CHANNEL_VALUE;
 	self->priv->scale_type = GTH_HISTOGRAM_SCALE_LINEAR;
