@@ -271,8 +271,12 @@ profile_ready_cb (GObject      *source_object,
 		GthICCProfile *profile;
 
 		profile = gth_color_manager_get_profile_finish (GTH_COLOR_MANAGER (source_object), res, NULL);
-		if (profile == NULL)
+		if (profile == NULL) {
 			profile = _g_object_ref (gth_browser_get_monitor_profile (self->priv->browser));
+		}
+		if (profile == NULL) {
+			profile = gth_icc_profile_new_srgb ();
+		}
 		gth_image_preloader_set_out_profile (self->priv->preloader, profile);
 
 		_gth_image_viewer_page_load_with_preloader_step2 (profile_data->self,
@@ -321,7 +325,10 @@ _gth_image_viewer_page_load_with_preloader (GthImageViewerPage  *self,
 		}
 	}
 
-	gth_image_preloader_set_out_profile (self->priv->preloader, NULL);
+	GthICCProfile *default_profile = self->priv->apply_icc_profile ? gth_icc_profile_new_srgb () : NULL;
+	gth_image_preloader_set_out_profile (self->priv->preloader, default_profile);
+	_g_object_unref (default_profile);
+
 	_gth_image_viewer_page_load_with_preloader_step2 (self, file_data, requested_size, cancellable, callback, user_data);
 }
 
@@ -1015,6 +1022,24 @@ pref_transparency_style_changed (GSettings *settings,
 }
 
 
+static void
+pref_show_frame_changed (GSettings *settings,
+			 char      *key,
+			 gpointer   user_data)
+{
+	GthImageViewerPage *self = user_data;
+
+	if (! self->priv->active || (self->priv->viewer == NULL))
+		return;
+
+	GthImageViewerTool *tool = gth_image_viewer_get_tool (GTH_IMAGE_VIEWER (self->priv->viewer));
+	if (! GTH_IS_IMAGE_DRAGGER (tool))
+		return;
+	gboolean show_frame = g_settings_get_boolean (self->priv->settings, PREF_IMAGE_VIEWER_SHOW_FRAME);
+	g_object_set (tool, "show-frame", show_frame, NULL);
+}
+
+
 static void
 paint_comment_over_image_func (GthImageViewer *image_viewer,
 			       cairo_t        *cr,
@@ -1296,12 +1321,12 @@ gth_image_viewer_page_real_activate (GthViewerPage *base,
 	gtk_widget_show (self->priv->image_navigator);
 
 	self->priv->overview_revealer = gtk_revealer_new ();
-	gtk_revealer_set_transition_duration (GTK_REVEALER (self->priv->overview_revealer), 200);
-	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->overview_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_DOWN);
+	gtk_revealer_set_transition_duration (GTK_REVEALER (self->priv->overview_revealer), 100);
+	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->overview_revealer), GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
 	gtk_widget_show (self->priv->overview_revealer);
 	gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->image_navigator), self->priv->overview_revealer);
 
-	self->priv->overview = gth_image_overview_new (GTH_IMAGE_VIEWER (self->priv->viewer));
+	self->priv->overview = gth_image_overview_new (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	gtk_widget_add_events (self->priv->overview, GDK_POINTER_MOTION_HINT_MASK | GDK_LEAVE_NOTIFY_MASK);
 	gtk_widget_show (self->priv->overview);
 	gtk_container_add (GTK_CONTAINER (self->priv->overview_revealer), self->priv->overview);
@@ -1460,18 +1485,11 @@ preloader_load_ready_cb (GObject	*source_object,
 					 original_height);
 
 	{
-		GthICCProfile *profile;
-
-		profile = gth_image_get_icc_profile (image);
-		if (profile != NULL) {
-			char *desc = gth_icc_profile_get_description (profile);
-
-			if (desc != NULL) {
-				g_file_info_set_attribute_string (self->priv->updated_info,
-								  "Loaded::Image::ColorProfile",
-								  desc);
-				g_free (desc);
-			}
+		const char *desc = gth_image_get_original_profile_name (image);
+		if (desc != NULL) {
+			g_file_info_set_attribute_string (self->priv->updated_info,
+							"Loaded::Image::ColorProfile",
+							desc);
 		}
 	}
 
@@ -1610,7 +1628,7 @@ gth_image_viewer_page_real_fullscreen (GthViewerPage *base,
 	if (! GTH_IS_IMAGE_DRAGGER (tool))
 		return;
 
-	g_object_set (tool, "show-frame", ! active, NULL);
+	g_object_set (tool, "show-frame", active ? FALSE : g_settings_get_boolean (self->priv->settings, PREF_IMAGE_VIEWER_SHOW_FRAME), NULL);
 }
 
 
@@ -2186,6 +2204,10 @@ gth_image_viewer_page_init (GthImageViewerPage *self)
 			  "changed::" PREF_IMAGE_VIEWER_TRANSPARENCY_STYLE,
 			  G_CALLBACK (pref_transparency_style_changed),
 			  self);
+	g_signal_connect (self->priv->settings,
+			  "changed::" PREF_IMAGE_VIEWER_SHOW_FRAME,
+			  G_CALLBACK (pref_show_frame_changed),
+			  self);
 }
 
 
@@ -2360,7 +2382,7 @@ gth_image_viewer_page_reset_viewer_tool	(GthImageViewerPage *self)
 {
 	GthImageViewerTool *dragger;
 
-	dragger = gth_image_dragger_new (TRUE);
+	dragger = gth_image_dragger_new (g_settings_get_boolean (self->priv->settings, PREF_IMAGE_VIEWER_SHOW_FRAME));
 	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (self->priv->viewer), dragger);
 	g_object_unref (dragger);
 
@@ -2557,7 +2579,7 @@ gth_image_viewer_page_get_original (GthImageViewerPage	 *self,
 gboolean
 gth_image_viewer_page_get_original_finish (GthImageViewerPage	 *self,
 					   GAsyncResult		 *result,
-					   cairo_surface_t	**image_p,
+					   GthImage		**image_p,
 					   GError		**error)
 {
 	GthImage *image;
@@ -2569,7 +2591,7 @@ gth_image_viewer_page_get_original_finish (GthImageViewerPage	 *self,
 		return FALSE;
 
 	if (image_p != NULL)
-		*image_p = gth_image_get_cairo_surface (image);
+		*image_p = g_object_ref (image);
 
 	g_object_unref (image);
 
@@ -2609,7 +2631,7 @@ get_original_image_ready_cb (GObject		*source_object,
 			    gpointer		 user_data)
 {
 	GthOriginalImageTask *self = user_data;
-	cairo_surface_t      *image = NULL;
+	GthImage             *image = NULL;
 	GError               *error = NULL;
 
 	if (gth_image_viewer_page_get_original_finish (self->viewer_page,
@@ -2617,11 +2639,11 @@ get_original_image_ready_cb (GObject		*source_object,
 						       &image,
 						       &error))
 	{
-		gth_image_task_set_destination_surface (GTH_IMAGE_TASK (self), image);
+		gth_image_task_set_destination (GTH_IMAGE_TASK (self), image);
 	}
 	gth_task_completed (GTH_TASK (self), error);
 
-	cairo_surface_destroy (image);
+	_g_object_unref (image);
 	_g_error_free (error);
 }
 
