@@ -25,9 +25,11 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
+#include "gio-utils.h"
 #include "glib-utils.h"
 #include "gth-file-source.h"
 #include "gth-location-chooser.h"
+#include "gth-location-chooser-dialog.h"
 #include "gth-main.h"
 #include "gtk-utils.h"
 #include "pixbuf-utils.h"
@@ -36,12 +38,13 @@
 #define MIN_WIDTH 200
 
 
-enum {
+typedef enum {
 	ITEM_TYPE_NONE,
 	ITEM_TYPE_SEPARATOR,
 	ITEM_TYPE_LOCATION,
-	ITEM_TYPE_ENTRY_POINT
-};
+	ITEM_TYPE_ENTRY_POINT,
+	ITEM_TYPE_CHOOSE_LOCATION
+} ItemType;
 
 enum {
 	ICON_COLUMN,
@@ -55,6 +58,8 @@ enum {
 enum {
 	PROP_0,
 	PROP_SHOW_ENTRY_POINTS,
+	PROP_SHOW_OTHER,
+	PROP_SHOW_ROOT,
 	PROP_RELIEF
 };
 
@@ -73,6 +78,8 @@ struct _GthLocationChooserPrivate {
 	guint           update_entry_list_id;
 	guint           update_location_list_id;
 	gboolean        show_entry_points;
+	gboolean        show_root;
+	gboolean        show_other;
 	GtkReliefStyle  relief;
 	gboolean        reload;
 };
@@ -101,6 +108,12 @@ gth_location_chooser_set_property (GObject      *object,
 	case PROP_SHOW_ENTRY_POINTS:
 		gth_location_chooser_set_show_entry_points (self, g_value_get_boolean (value));
 		break;
+	case PROP_SHOW_ROOT:
+		self->priv->show_root = g_value_get_boolean (value);
+		break;
+	case PROP_SHOW_OTHER:
+		self->priv->show_other = g_value_get_boolean (value);
+		break;
 	case PROP_RELIEF:
 		gth_location_chooser_set_relief (self, g_value_get_enum (value));
 		break;
@@ -124,6 +137,12 @@ gth_location_chooser_get_property (GObject    *object,
 	case PROP_SHOW_ENTRY_POINTS:
 		g_value_set_boolean (value, self->priv->show_entry_points);
 		break;
+	case PROP_SHOW_ROOT:
+		g_value_set_boolean (value, self->priv->show_root);
+		break;
+	case PROP_SHOW_OTHER:
+		g_value_set_boolean (value, self->priv->show_other);
+		break;
 	case PROP_RELIEF:
 		g_value_set_enum (value, self->priv->relief);
 		break;
@@ -193,6 +212,44 @@ get_nth_separator_pos (GthLocationChooser *self,
 }
 
 
+static gboolean
+get_iter_from_current_file_entries (GthLocationChooser *self,
+				    GFile              *file,
+				    GtkTreeIter        *iter)
+{
+	gboolean  found = FALSE;
+	char     *uri;
+
+	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->model), iter))
+		return FALSE;
+
+	uri = g_file_get_uri (file);
+	do {
+		int   item_type = ITEM_TYPE_NONE;
+		char *list_uri;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
+				    iter,
+				    TYPE_COLUMN, &item_type,
+				    URI_COLUMN, &list_uri,
+				    -1);
+		if (item_type == ITEM_TYPE_SEPARATOR)
+			break;
+		if (_g_str_equal (uri, list_uri)) {
+			found = TRUE;
+			g_free (list_uri);
+			break;
+		}
+		g_free (list_uri);
+	}
+	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->model), iter));
+
+	g_free (uri);
+
+	return found;
+}
+
+
 static void
 combo_changed_cb (GtkComboBox *widget,
 		  gpointer     user_data)
@@ -211,7 +268,27 @@ combo_changed_cb (GtkComboBox *widget,
 			    URI_COLUMN, &uri,
 			    -1);
 
-	if (uri != NULL) {
+	if (item_type == ITEM_TYPE_CHOOSE_LOCATION) {
+		GtkWidget *dialog;
+
+		dialog = gth_location_chooser_dialog_new (_("Location"), _gtk_widget_get_toplevel_if_window (GTK_WIDGET (widget)));
+		if (self->priv->location != NULL)
+			gth_location_chooser_dialog_set_folder (GTH_LOCATION_CHOOSER_DIALOG (dialog), self->priv->location);
+
+		switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
+		case GTK_RESPONSE_OK:
+			gth_location_chooser_set_current (self, gth_location_chooser_dialog_get_folder (GTH_LOCATION_CHOOSER_DIALOG (dialog)));
+			break;
+
+		default:
+			/* reset the previous value. */
+			gth_location_chooser_set_current (self, self->priv->location);
+			break;
+		}
+
+		gtk_widget_destroy (dialog);
+	}
+	else if (uri != NULL) {
 		GFile *file;
 
 		file = g_file_new_for_uri (uri);
@@ -256,12 +333,14 @@ add_file_source_entries (GthLocationChooser *self,
 
 
 static void
-clear_entry_point_list (GthLocationChooser *self)
+clear_items_from_separator (GthLocationChooser *self,
+			    int                 nth_separator,
+			    gboolean            stop_at_next_separator)
 {
 	int first_position;
 	int i;
 
-	if (! get_nth_separator_pos (self, 1, &first_position))
+	if (! get_nth_separator_pos (self, nth_separator, &first_position))
 		return;
 
 	for (i = first_position + 1; TRUE; i++) {
@@ -269,8 +348,19 @@ clear_entry_point_list (GthLocationChooser *self)
 		GtkTreeIter  iter;
 
 		path = gtk_tree_path_new_from_indices (first_position + 1, -1);
-		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->model), &iter, path))
+		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (self->priv->model), &iter, path)) {
+			if (stop_at_next_separator) {
+				int item_type = ITEM_TYPE_NONE;
+
+				gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
+						    &iter,
+						    TYPE_COLUMN, &item_type,
+						    -1);
+				if (item_type == ITEM_TYPE_SEPARATOR)
+					break;
+			}
 			gtk_tree_store_remove (self->priv->model, &iter);
+		}
 		else
 			break;
 
@@ -279,36 +369,101 @@ clear_entry_point_list (GthLocationChooser *self)
 }
 
 
+static void
+delete_section_by_type (GthLocationChooser *self,
+			ItemType            type_to_delete)
+{
+	GtkTreeIter iter;
+	gboolean    valid;
+	gboolean    prev_matched;
+
+	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->model), &iter))
+		return;
+
+	prev_matched = FALSE;
+	do {
+		int item_type = ITEM_TYPE_NONE;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
+				    &iter,
+				    TYPE_COLUMN, &item_type,
+				    -1);
+
+		if (item_type == type_to_delete) {
+			valid = gtk_tree_store_remove (self->priv->model, &iter);
+			prev_matched = TRUE;
+		}
+		else {
+			if (prev_matched && (item_type == ITEM_TYPE_SEPARATOR))
+				valid = gtk_tree_store_remove (self->priv->model, &iter);
+			else
+				valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->model), &iter);
+			prev_matched = FALSE;
+		}
+	}
+	while (valid);
+}
+
+
+static gboolean
+get_section_end_by_type (GthLocationChooser *self,
+			 ItemType            item_to_search,
+			 int                *p_pos)
+{
+	GtkTreeIter iter;
+	int         pos;
+	gboolean    valid;
+	gboolean    prev_matched;
+
+	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->model), &iter))
+		return FALSE;
+
+	pos = 0;
+	valid = FALSE;
+	prev_matched = FALSE;
+	do {
+		int item_type = ITEM_TYPE_NONE;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
+				    &iter,
+				    TYPE_COLUMN, &item_type,
+				    -1);
+
+		if (item_type == item_to_search) {
+			*p_pos = pos;
+			valid = TRUE;
+			prev_matched = TRUE;
+		}
+		else {
+			if ((item_type == ITEM_TYPE_SEPARATOR) && prev_matched)
+				*p_pos = pos;
+			prev_matched = FALSE;
+		}
+
+		pos++;
+	}
+	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->model), &iter));
+
+	return valid;
+}
+
+
 static void
 update_entry_point_list (GthLocationChooser *self)
 {
-	int    first_position;
 	int    position;
 	GList *entry_points;
 	GList *scan;
 
 	self->priv->update_entry_list_id = 0;
 
-	clear_entry_point_list (self);
+	delete_section_by_type (self, ITEM_TYPE_ENTRY_POINT);
 
-	if (! get_nth_separator_pos (self, 1, &first_position)) {
-		GtkTreeIter  iter;
-		GtkTreePath *path;
+	if (get_section_end_by_type (self, ITEM_TYPE_LOCATION, &position))
+		position = position + 1;
+	else
+		position = 0;
 
-		gtk_tree_store_append (self->priv->model, &iter, NULL);
-		gtk_tree_store_set (self->priv->model, &iter,
-				    TYPE_COLUMN, ITEM_TYPE_SEPARATOR,
-				    -1);
-
-		path = gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->model), &iter);
-		if (path == NULL)
-			return;
-		first_position = gtk_tree_path_get_indices(path)[0];
-
-		gtk_tree_path_free (path);
-	}
-
-	position = first_position + 1;
 	entry_points = gth_main_get_all_entry_points ();
 	for (scan = entry_points; scan; scan = scan->next) {
 		GthFileData *file_data = scan->data;
@@ -322,6 +477,15 @@ update_entry_point_list (GthLocationChooser *self)
 					 ITEM_TYPE_ENTRY_POINT);
 	}
 
+	if (self->priv->show_other) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_insert (self->priv->model, &iter, NULL, position);
+		gtk_tree_store_set (self->priv->model, &iter,
+				    TYPE_COLUMN, ITEM_TYPE_SEPARATOR,
+				    -1);
+	}
+
 	_g_object_list_unref (entry_points);
 }
 
@@ -354,69 +518,6 @@ entry_points_changed_cb (GthMonitor         *monitor,
 }
 
 
-static gboolean
-delete_current_file_entries (GthLocationChooser *self)
-{
-	gboolean    found = FALSE;
-	GtkTreeIter iter;
-
-	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->model), &iter))
-		return FALSE;
-
-	do {
-		int item_type = ITEM_TYPE_NONE;
-
-		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
-				    &iter,
-				    TYPE_COLUMN, &item_type,
-				    -1);
-		if (item_type == ITEM_TYPE_SEPARATOR)
-			break;
-	}
-	while (gtk_tree_store_remove (self->priv->model, &iter));
-
-	return found;
-}
-
-
-static gboolean
-get_iter_from_current_file_entries (GthLocationChooser *self,
-				    GFile              *file,
-				    GtkTreeIter        *iter)
-{
-	gboolean  found = FALSE;
-	char     *uri;
-
-	if (! gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->priv->model), iter))
-		return FALSE;
-
-	uri = g_file_get_uri (file);
-	do {
-		int   item_type = ITEM_TYPE_NONE;
-		char *list_uri;
-
-		gtk_tree_model_get (GTK_TREE_MODEL (self->priv->model),
-				    iter,
-				    TYPE_COLUMN, &item_type,
-				    URI_COLUMN, &list_uri,
-				    -1);
-		if (item_type == ITEM_TYPE_SEPARATOR)
-			break;
-		if (same_uri (uri, list_uri)) {
-			found = TRUE;
-			g_free (list_uri);
-			break;
-		}
-		g_free (list_uri);
-	}
-	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (self->priv->model), iter));
-
-	g_free (uri);
-
-	return found;
-}
-
-
 static void
 update_location_list (gpointer user_data)
 {
@@ -438,16 +539,17 @@ update_location_list (gpointer user_data)
 		GList *scan;
 		int    position = 0;
 
-		delete_current_file_entries (self);
+		delete_section_by_type (self, ITEM_TYPE_LOCATION);
 
 		list = gth_file_source_get_current_list (self->priv->file_source, self->priv->location);
 		for (scan = list; scan; scan = scan->next) {
 			GFile     *file = scan->data;
 			GFileInfo *info;
 
-			info = gth_file_source_get_file_info (self->priv->file_source, file, GFILE_DISPLAY_ATTRIBUTES);
+			info = _g_file_get_info_for_display (file);
 			if (info == NULL)
 				continue;
+
 			add_file_source_entries (self,
 						 file,
 						 g_file_info_get_display_name (info),
@@ -459,6 +561,30 @@ update_location_list (gpointer user_data)
 			g_object_unref (info);
 		}
 
+		if (self->priv->show_root) {
+			GIcon *icon;
+
+			icon = g_themed_icon_new ("computer-symbolic");
+			gtk_tree_store_insert (self->priv->model, &iter, NULL, position++);
+			gtk_tree_store_set (self->priv->model, &iter,
+					    TYPE_COLUMN, ITEM_TYPE_LOCATION,
+					    ICON_COLUMN, icon,
+					    NAME_COLUMN, _("Locations"),
+					    URI_COLUMN, "gthumb-vfs:///",
+					    -1);
+
+			_g_object_unref (icon);
+		}
+
+		if (self->priv->show_other || self->priv->show_entry_points) {
+			GtkTreeIter iter;
+
+			gtk_tree_store_insert (self->priv->model, &iter, NULL, position);
+			gtk_tree_store_set (self->priv->model, &iter,
+					    TYPE_COLUMN, ITEM_TYPE_SEPARATOR,
+					    -1);
+		}
+
 		_g_object_list_unref (list);
 	}
 
@@ -481,6 +607,17 @@ gth_location_chooser_realize (GtkWidget *widget)
 	GthLocationChooser *self = GTH_LOCATION_CHOOSER (widget);
 
 	GTK_WIDGET_CLASS (gth_location_chooser_parent_class)->realize (widget);
+
+	if (self->priv->show_other) {
+		GtkTreeIter iter;
+
+		gtk_tree_store_append (self->priv->model, &iter, NULL);
+		gtk_tree_store_set (self->priv->model, &iter,
+				    TYPE_COLUMN, ITEM_TYPE_CHOOSE_LOCATION,
+				    NAME_COLUMN, _("Other…"),
+				    ELLIPSIZE_COLUMN, FALSE,
+				    -1);
+	}
 	entry_points_changed_cb (NULL, self);
 	current_location_changed (self);
 }
@@ -505,11 +642,24 @@ gth_location_chooser_class_init (GthLocationChooserClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_ENTRY_POINTS,
 					 g_param_spec_boolean ("show-entry-points",
-                                                               "Show entry points",
-                                                               "Whether to show the entry points in the list",
-                                                               TRUE,
-                                                               G_PARAM_READWRITE));
-
+							       "Show entry points",
+							       "Whether to show the entry points in the list",
+							       TRUE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_ROOT,
+					 g_param_spec_boolean ("show-root",
+							       "Show the VFS root",
+							       "Whether to show the VFS root in the list",
+							       FALSE,
+							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_OTHER,
+					 g_param_spec_boolean ("show-other",
+							       "Show the Other... entry",
+							       "Whether to show a special entry to choose a location from a dialog",
+							       FALSE,
+							       G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_RELIEF,
 					 g_param_spec_enum ("relief",
@@ -544,6 +694,8 @@ gth_location_chooser_init (GthLocationChooser *self)
 	self->priv = gth_location_chooser_get_instance_private (self);
 	self->priv->entry_points_changed_id = 0;
 	self->priv->show_entry_points = TRUE;
+	self->priv->show_root = FALSE;
+	self->priv->show_other = TRUE;
 	self->priv->relief = GTK_RELIEF_NORMAL;
 	self->priv->reload = FALSE;
 
@@ -647,7 +799,7 @@ gth_location_chooser_set_show_entry_points (GthLocationChooser *self,
 			g_source_remove (self->priv->entry_points_changed_id);
 			self->priv->entry_points_changed_id = 0;
 		}
-		clear_entry_point_list (self);
+		clear_items_from_separator (self, 1, TRUE);
 	}
 
 	g_object_notify (G_OBJECT (self), "show-entry-points");
@@ -665,6 +817,9 @@ void
 gth_location_chooser_set_current (GthLocationChooser *self,
 				  GFile              *file)
 {
+	if (file == NULL)
+		return;
+
 	if (file != self->priv->location) {
 		if (self->priv->file_source != NULL)
 			g_object_unref (self->priv->file_source);
