@@ -36,6 +36,7 @@ gth_metadata_provider_file_can_read (GthMetadataProvider  *self,
 {
 	return _g_file_attributes_matches_any_v ("gth::file::display-size,"
 						 "gth::file::display-mtime,"
+						 "gth::file::display-ctime,"
 						 "gth::file::content-type,"
 						 "gth::file::is-modified,"
 						 "gth::file::full-name",
@@ -49,12 +50,9 @@ gth_metadata_provider_file_read (GthMetadataProvider *self,
 				 const char          *attributes,
 				 GCancellable        *cancellable)
 {
-	GFileAttributeMatcher *matcher;
-	char                  *value;
-	GTimeVal              *timeval_p;
-	const char            *value_s;
-
-	matcher = g_file_attribute_matcher_new (attributes);
+	char       *value;
+	GTimeVal   *timeval_p;
+	const char *value_s;
 
 	value = g_format_size (g_file_info_get_size (file_data->info));
 	g_file_info_set_attribute_string (file_data->info, "gth::file::display-size", value);
@@ -65,6 +63,11 @@ gth_metadata_provider_file_read (GthMetadataProvider *self,
 	g_file_info_set_attribute_string (file_data->info, "gth::file::display-mtime", value);
 	g_free (value);
 
+	timeval_p = gth_file_data_get_creation_time (file_data);
+	value = _g_time_val_strftime (timeval_p, "%x %X");
+	g_file_info_set_attribute_string (file_data->info, "gth::file::display-ctime", value);
+	g_free (value);
+
 	value = g_file_get_parse_name (file_data->file);
 	g_file_info_set_attribute_string (file_data->info, "gth::file::full-name", value);
 	g_free (value);
@@ -72,8 +75,6 @@ gth_metadata_provider_file_read (GthMetadataProvider *self,
 	value_s = _g_str_get_static (_g_file_info_get_content_type (file_data->info));
 	if (value_s != NULL)
 		g_file_info_set_attribute_string (file_data->info, "gth::file::content-type", value_s);
-
-	g_file_attribute_matcher_unref (matcher);
 }
 
 
